@@ -8,15 +8,13 @@
 #ifndef __FEMONSERVICE_H
 #define __FEMONSERVICE_H
 
-#include <linux/dvb/frontend.h>
-
-struct FemonService_v1_0 {
+struct FemonService_v1_1 {
   cString fe_name;
   cString fe_status;
-  uint16_t fe_snr;
-  uint16_t fe_signal;
-  uint32_t fe_ber;
-  uint32_t fe_unc;
+  double fe_cnr;
+  double fe_signal;
+  double fe_ber;
+  double fe_per;
   double video_bitrate;
   double audio_bitrate;
   double dolby_bitrate;
